import { Permission, PermissionStatus, Rationale } from 'react-native';
import { Indentifier, MEDIA_MIME_TYPE, Result, Settings } from './types';
/**
 * Create deck, model, and references. Once set up, notes can be created.
 *  All newly created notes must have the correct matching info.
 * - only one needed of (deckId or deckPoperties) and (modelId or modelProperties)
 * - Will retry once if there is an error. Sometimes the `AnkiDroid` native api
 * will return an error the first time but will work fine on the second attempt
 * @constructor settings object with the below values
 * - deckId: required if `deckProperties` missing
 * - deckProperties: required if `deckId` missing
 * - modelId: required if `modelProperties` missing
 * - modelProperties: required if `modelId` missing
 */
export declare class AnkiDroid {
    settings: Settings;
    constructor(settings: Settings);
    /**
     * Check if android
     * - display error message if not android
     * @returns `true` if android
     */
    static androidCheck(): boolean;
    /**
     * Get the AnkiDroid API permission name
     */
    static getPermissionName(): Promise<Permission>;
    /**
     * Check if the AnkiDroid API is available on the phone
     * @return `true` if the API is available to use
     */
    static isApiAvailable(): Promise<boolean>;
    /**
     * Check the permission status
     * @return `true` if permission have been granted
     */
    static checkPermission(): Promise<boolean>;
    /**
     * Request AnkiDroid API permissions
     * @param rationale optional `PermissionsAndroid` message to show when requesting permissions
     * @return a tuple of any errors and the result `[error, result]`
     */
    static requestPermission(rationale?: Rationale): Promise<Result<PermissionStatus>>;
    /**
     * Gets the ID and name for all decks
     * @return a tuple of any errors and the result `[error, result]`
     */
    static getDeckList(): Promise<Result<Indentifier[]>>;
    /**
     * Private method with the logic
     */
    private static _getDeckList;
    /**
     * Gets the ID and name for all models
     * @return a tuple of any errors and the result `[error, result]`
     */
    static getModelList(): Promise<Result<Indentifier[]>>;
    /**
     * Private method with the logic
     */
    private static _getModelList;
    /**
     * Gets all field names for a specific model
     * @param modelName required if `modelId` is not used
     * @param modelId required if `modelName` is not used
     * @return a tuple of any errors and the result `[error, result]`
     */
    static getFieldList(modelName?: string, modelId?: number | string): Promise<Result<string[]>>;
    /**
     * Private method with the logic
     */
    private static _getFieldList;
    /**
     * Gets the name of the currently selected deck
     * @return a tuple of any errors and the result `[error, result]`
     */
    static getSelectedDeckName(): Promise<Result<string>>;
    /**
     * Private method with the logic
     */
    private static _getSelectedDeckName;
    /**
     * Upload media from the URI and get the correctly formatted string for the
     * media file to be placed in the desired field of a card
     *
     * Notes to consider:
     * * Both AnkiDroid and your app should have the
     * `android.permission.MANAGE_EXTERNAL_STORAGE` permission granted if you wish
     * to upload the file from external storage
     *
     * @return a tuple of any errors and the result `[error, result]`
     */
    static uploadMediaFromUri(fileUri: string, preferredName: string, mimeType: MEDIA_MIME_TYPE): Promise<Result<string>>;
    /**
     * Private method with the logic
     */
    private static _uploadMediaFromUri;
    /**
     * check all note values for errors
     * @param note
     * @returns `null` if no errors
     */
    private checkForAddNoteErrors;
    /**
     * check all property values for errors
     * @param properties
     * @returns `null` if no errors
     */
    private checkForPropertyErrors;
    private checkIfModelFieldsAreTheSame;
    /**
     * checks if the card fields are valid types and that they have the same length
     * @param modelFields
     * @param valueFields
     */
    private checkValidFields;
    /**
     * checks that arrays are the correct length
     * @param noteValue
     * @param noteKey
     */
    private checkArrayLength;
    /**
     * logs errors
     * @param noteKey
     */
    private logTypeError;
    /**
     * gets valid id and property values
     * @param id deckId | modelId
     * @param properties deckProperties | modelProperties
     * @returns `Error` if not valid
     */
    private getValidIdAndProperties;
    /**
     * Checks if a valid string or array of strings
     * @param itemToCheck
     * @returns `true` if valid
     */
    private checkValidString;
    /**
     * Private method with the logic
     */
    private _addNote;
    /**
     * Create notes using the created deck model
     * @param valueFields length must match `modelFields`
     * @param modelFields length must match the settings used when creating the deck
     * @return a tuple of any errors and the result `[error, result]`
     * @return the added note ID
     */
    addNote(valueFields: string[], modelFields: string[]): Promise<Result<string>>;
}
