var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : new P(function (resolve) { resolve(result.value); }).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
import { NativeModules, PermissionsAndroid, Platform, } from 'react-native';
import { Errors, ErrorText, MODULE_NAME, NoteKeys, } from './types';
const { AnkiDroidModule } = NativeModules;
/**
 * Create deck, model, and references. Once set up, notes can be created.
 *  All newly created notes must have the correct matching info.
 * - only one needed of (deckId or deckPoperties) and (modelId or modelProperties)
 * - Will retry once if there is an error. Sometimes the `AnkiDroid` native api
 * will return an error the first time but will work fine on the second attempt
 * @constructor settings object with the below values
 * - deckId: required if `deckProperties` missing
 * - deckProperties: required if `deckId` missing
 * - modelId: required if `modelProperties` missing
 * - modelProperties: required if `modelId` missing
 */
export class AnkiDroid {
    constructor(settings) {
        this.settings = settings;
    }
    ////////////
    // STATIC //
    ////////////
    /**
     * Check if android
     * - display error message if not android
     * @returns `true` if android
     */
    static androidCheck() {
        if (Platform.OS === 'android') {
            return true;
        }
        console.warn(MODULE_NAME, Errors.OS_ERROR);
        return false;
    }
    /**
     * Get the AnkiDroid API permission name
     */
    static getPermissionName() {
        return __awaiter(this, void 0, void 0, function* () {
            let permissionName;
            try {
                permissionName = yield AnkiDroidModule.getPermissionName();
            }
            catch (error) {
                permissionName = null;
                console.warn(MODULE_NAME, ErrorText.PERMISSION_NAME, error);
            }
            return permissionName;
        });
    }
    /**
     * Check if the AnkiDroid API is available on the phone
     * @return `true` if the API is available to use
     */
    static isApiAvailable() {
        return __awaiter(this, void 0, void 0, function* () {
            if (!AnkiDroid.androidCheck())
                return false;
            let apiAvailable;
            try {
                apiAvailable = yield AnkiDroidModule.isApiAvailable();
            }
            catch (error) {
                apiAvailable = false;
                console.warn(MODULE_NAME, ErrorText.API, error);
            }
            return apiAvailable;
        });
    }
    /**
     * Check the permission status
     * @return `true` if permission have been granted
     */
    static checkPermission() {
        return __awaiter(this, void 0, void 0, function* () {
            if (!AnkiDroid.androidCheck())
                return false;
            let permissionName;
            try {
                permissionName = yield AnkiDroid.getPermissionName();
            }
            catch (error) {
                console.warn(MODULE_NAME, error.toString());
                return false;
            }
            if (!permissionName)
                return false;
            try {
                const permission = yield PermissionsAndroid.check(permissionName);
                return permission;
            }
            catch (error) {
                console.warn(MODULE_NAME, ErrorText.PERMISSIONS_CHECK, error);
                return false;
            }
        });
    }
    /**
     * Request AnkiDroid API permissions
     * @param rationale optional `PermissionsAndroid` message to show when requesting permissions
     * @return a tuple of any errors and the result `[error, result]`
     */
    static requestPermission(rationale = null) {
        return __awaiter(this, void 0, void 0, function* () {
            if (!AnkiDroid.androidCheck())
                return [new Error(Errors.OS_ERROR)];
            let permissionName;
            try {
                permissionName = yield AnkiDroid.getPermissionName();
            }
            catch (error) {
                console.warn(MODULE_NAME, error.toString());
                return [new Error(Errors.UNKNOWN_ERROR)];
            }
            if (!permissionName)
                return [null, 'denied'];
            try {
                const permissionRequest = (yield PermissionsAndroid.request(permissionName, rationale));
                return [null, permissionRequest];
            }
            catch (error) {
                console.warn(MODULE_NAME, ErrorText.PERMISSIONS_REQUEST, error);
                return [new Error(error.toString())];
            }
        });
    }
    /**
     * Gets the ID and name for all decks
     * @return a tuple of any errors and the result `[error, result]`
     */
    static getDeckList() {
        return __awaiter(this, void 0, void 0, function* () {
            const [error, response] = yield AnkiDroid._getDeckList();
            if (error) {
                return yield AnkiDroid._getDeckList();
            }
            return [error, response];
        });
    }
    /**
     * Private method with the logic
     */
    static _getDeckList() {
        return __awaiter(this, void 0, void 0, function* () {
            if (!AnkiDroid.androidCheck())
                return [new Error(Errors.OS_ERROR)];
            const permissionStatus = yield AnkiDroid.checkPermission();
            if (!permissionStatus)
                return [new Error(Errors.PERMISSION_ERROR)];
            try {
                const decks = yield AnkiDroidModule.getDeckList();
                return [null, decks];
            }
            catch (error) {
                console.warn(MODULE_NAME, error.toString());
                return [new Error(Errors.UNKNOWN_ERROR)];
            }
        });
    }
    /**
     * Gets the ID and name for all models
     * @return a tuple of any errors and the result `[error, result]`
     */
    static getModelList() {
        return __awaiter(this, void 0, void 0, function* () {
            const [error, response] = yield AnkiDroid._getModelList();
            if (error) {
                return yield AnkiDroid._getModelList();
            }
            return [error, response];
        });
    }
    /**
     * Private method with the logic
     */
    static _getModelList() {
        return __awaiter(this, void 0, void 0, function* () {
            if (!AnkiDroid.androidCheck())
                return [new Error(Errors.OS_ERROR)];
            const permissionStatus = yield AnkiDroid.checkPermission();
            if (!permissionStatus)
                return [new Error(Errors.PERMISSION_ERROR)];
            try {
                const models = yield AnkiDroidModule.getModelList();
                return [null, models];
            }
            catch (error) {
                console.warn(MODULE_NAME, error.toString());
                return [new Error(Errors.UNKNOWN_ERROR)];
            }
        });
    }
    /**
     * Gets all field names for a specific model
     * @param modelName required if `modelId` is not used
     * @param modelId required if `modelName` is not used
     * @return a tuple of any errors and the result `[error, result]`
     */
    static getFieldList(modelName, modelId) {
        return __awaiter(this, void 0, void 0, function* () {
            const [error, response] = yield AnkiDroid._getFieldList(modelName, modelId);
            if (error) {
                return yield AnkiDroid._getFieldList(modelName, modelId);
            }
            return [error, response];
        });
    }
    /**
     * Private method with the logic
     */
    static _getFieldList(modelName, modelId) {
        return __awaiter(this, void 0, void 0, function* () {
            if (!AnkiDroid.androidCheck())
                return [new Error(Errors.OS_ERROR)];
            const permissionStatus = yield AnkiDroid.checkPermission();
            if (!permissionStatus)
                return [new Error(Errors.PERMISSION_ERROR)];
            if (!modelName && !modelId)
                return [new Error(Errors.IDENTIFIER_MISSING)];
            if (typeof modelId === 'number') {
                modelId = modelId.toString();
            }
            try {
                const fieldList = yield AnkiDroidModule.getFieldList(modelName || null, modelId || null);
                return [null, fieldList];
            }
            catch (error) {
                console.warn(MODULE_NAME, error.toString());
                return [new Error(Errors.UNKNOWN_ERROR)];
            }
        });
    }
    /**
     * Gets the name of the currently selected deck
     * @return a tuple of any errors and the result `[error, result]`
     */
    static getSelectedDeckName() {
        return __awaiter(this, void 0, void 0, function* () {
            const [error, response] = yield AnkiDroid._getSelectedDeckName();
            if (error) {
                return yield AnkiDroid._getSelectedDeckName();
            }
            return [error, response];
        });
    }
    /**
     * Private method with the logic
     */
    static _getSelectedDeckName() {
        return __awaiter(this, void 0, void 0, function* () {
            if (!AnkiDroid.androidCheck())
                return [new Error(Errors.OS_ERROR)];
            const permissionStatus = yield AnkiDroid.checkPermission();
            if (!permissionStatus)
                return [new Error(Errors.PERMISSION_ERROR)];
            try {
                const deckName = yield AnkiDroidModule.getSelectedDeckName();
                return [null, deckName];
            }
            catch (error) {
                console.warn(MODULE_NAME, error.toString());
                return [new Error(Errors.UNKNOWN_ERROR)];
            }
        });
    }
    /**
     * Upload media from the URI and get the correctly formatted string for the
     * media file to be placed in the desired field of a card
     *
     * Notes to consider:
     * * Both AnkiDroid and your app should have the
     * `android.permission.MANAGE_EXTERNAL_STORAGE` permission granted if you wish
     * to upload the file from external storage
     *
     * @return a tuple of any errors and the result `[error, result]`
     */
    static uploadMediaFromUri(fileUri, preferredName, mimeType) {
        return __awaiter(this, void 0, void 0, function* () {
            const [error, response] = yield AnkiDroid._uploadMediaFromUri(fileUri, preferredName, mimeType);
            if (error) {
                return yield AnkiDroid._uploadMediaFromUri(fileUri, preferredName, mimeType);
            }
            return [error, response];
        });
    }
    /**
     * Private method with the logic
     */
    static _uploadMediaFromUri(fileUri, preferredName, mimeType) {
        return __awaiter(this, void 0, void 0, function* () {
            if (!AnkiDroid.androidCheck())
                return [new Error(Errors.OS_ERROR)];
            const permissionStatus = yield AnkiDroid.checkPermission();
            if (!permissionStatus)
                return [new Error(Errors.PERMISSION_ERROR)];
            try {
                const formatMediaName = yield AnkiDroidModule.uploadMediaFromUri(fileUri, preferredName, mimeType);
                return [null, formatMediaName];
            }
            catch (error) {
                console.warn(MODULE_NAME, error.toString());
                return [new Error(Errors.UNKNOWN_ERROR)];
            }
        });
    }
    /////////////
    // PRIVATE //
    /////////////
    /**
     * check all note values for errors
     * @param note
     * @returns `null` if no errors
     */
    checkForAddNoteErrors(note) {
        const { modelFields, valueFields } = note;
        if (!this.checkValidFields(modelFields, valueFields))
            return Errors.TYPE_ERROR;
        for (var key in note) {
            // skip loop if the property is from prototype
            if (!note.hasOwnProperty(key))
                continue;
            // check for null exceptions
            if (note[key] === null &&
                (key === NoteKeys.tags || key === NoteKeys.css)) {
                continue;
            }
            if (!this.checkArrayLength(note[key], key)) {
                return Errors.TYPE_ERROR;
            }
            if (!this.checkValidString(note[key])) {
                this.logTypeError(key);
                return Errors.TYPE_ERROR;
            }
        }
        return null;
    }
    /**
     * check all property values for errors
     * @param properties
     * @returns `null` if no errors
     */
    checkForPropertyErrors(properties) {
        for (var key in properties) {
            // skip loop if the property is from prototype
            if (!properties.hasOwnProperty(key))
                continue;
            // check for keys that can have a default null value
            if (!properties[key] && (key === NoteKeys.tags || key === NoteKeys.css)) {
                continue;
            }
            if (!this.checkArrayLength(properties[key], key)) {
                return Errors.TYPE_ERROR;
            }
            if (!this.checkValidString(properties[key])) {
                this.logTypeError(key);
                return Errors.TYPE_ERROR;
            }
        }
        return null;
    }
    checkIfModelFieldsAreTheSame(modelFields, modelFieldsFromNote) {
        let isSame = true;
        if (modelFields.length !== modelFieldsFromNote.length &&
            Array.isArray(modelFields) &&
            Array.isArray(modelFieldsFromNote)) {
            isSame = false;
        }
        if (isSame) {
            const compareSet = new Set(modelFields);
            modelFieldsFromNote.forEach(field => {
                compareSet.add(field);
            });
            if (compareSet.size !== modelFields.length) {
                isSame = false;
            }
        }
        if (!isSame) {
            console.warn(MODULE_NAME, ErrorText.ARGUMENT_TYPE, ErrorText.MODEL_FIELDS_DIFFERENT);
            return Errors.TYPE_ERROR;
        }
        return null;
    }
    /**
     * checks if the card fields are valid types and that they have the same length
     * @param modelFields
     * @param valueFields
     */
    checkValidFields(modelFields, valueFields) {
        try {
            if (modelFields.length !== valueFields.length &&
                Array.isArray(modelFields) &&
                Array.isArray(valueFields)) {
                console.warn(MODULE_NAME, ErrorText.ARGUMENT_TYPE, ErrorText.ARRAY_SAME_LENGTH);
                return false;
            }
        }
        catch (error) {
            console.warn(MODULE_NAME, ErrorText.ARGUMENT_TYPE, ErrorText.ARRAY_SAME_LENGTH, error.toString());
            return false;
        }
        return true;
    }
    /**
     * checks that arrays are the correct length
     * @param noteValue
     * @param noteKey
     */
    checkArrayLength(noteValue, noteKey) {
        try {
            switch (noteKey) {
                case NoteKeys.cardNames:
                    if (noteValue.length === 2 && Array.isArray(noteValue))
                        return true;
                    break;
                case NoteKeys.questionFormat:
                    if (noteValue.length === 2 && Array.isArray(noteValue))
                        return true;
                    break;
                case NoteKeys.answerFormat:
                    if (noteValue.length === 2 && Array.isArray(noteValue))
                        return true;
                    break;
                default:
                    return true;
            }
            console.warn(MODULE_NAME, ErrorText.ARGUMENT_TYPE, `${noteKey} ${ErrorText.ARRAY_LENGTH_2}`);
        }
        catch (error) {
            console.warn(MODULE_NAME, ErrorText.ARGUMENT_TYPE, `${noteKey} ${ErrorText.ARRAY_LENGTH_2}`, error.toString());
            return false;
        }
        return false;
    }
    /**
     * logs errors
     * @param noteKey
     */
    logTypeError(noteKey) {
        let errorText = null;
        switch (noteKey) {
            case NoteKeys.deckName:
                errorText = ErrorText.STRING;
                break;
            case NoteKeys.modelName:
                errorText = ErrorText.STRING;
                break;
            case NoteKeys.dbDeckReference:
                errorText = ErrorText.STRING;
                break;
            case NoteKeys.dbModelReference:
                errorText = ErrorText.STRING;
                break;
            case NoteKeys.modelFields:
                errorText = ErrorText.ARRAY_OF_STRING;
                break;
            case NoteKeys.valueFields:
                errorText = ErrorText.ARRAY_OF_STRING;
                break;
            case NoteKeys.tags:
                errorText = ErrorText.ARRAY_OF_STRING_OR_NULL;
                break;
            case NoteKeys.cardNames:
                errorText = ErrorText.ARRAY_OF_STRING;
                break;
            case NoteKeys.questionFormat:
                errorText = ErrorText.ARRAY_OF_STRING;
                break;
            case NoteKeys.answerFormat:
                errorText = ErrorText.ARRAY_OF_STRING;
                break;
            case NoteKeys.css:
                errorText = ErrorText.STRING_OR_NULL;
                break;
            default:
                errorText = ErrorText.NOTE_UNKNOWN;
                break;
        }
        console.warn(MODULE_NAME, ErrorText.ARGUMENT_TYPE, `${noteKey} ${errorText}`);
    }
    /**
     * gets valid id and property values
     * @param id deckId | modelId
     * @param properties deckProperties | modelProperties
     * @returns `Error` if not valid
     */
    getValidIdAndProperties(id, properties) {
        if (!id && !properties) {
            return new Error(ErrorText.DECK_INFO_MISSING);
        }
        else if (!id) {
            const errorCheckResults = this.checkForPropertyErrors(properties);
            if (errorCheckResults)
                return new Error(errorCheckResults);
        }
        else if (id) {
            if (typeof id === 'number') {
                id = id.toString();
            }
        }
        return [id, properties];
    }
    /**
     * Checks if a valid string or array of strings
     * @param itemToCheck
     * @returns `true` if valid
     */
    checkValidString(itemToCheck) {
        if (Array.isArray(itemToCheck)) {
            return itemToCheck.every(item => this.checkValidString(item));
        }
        else {
            return typeof itemToCheck === 'string';
        }
    }
    /**
     * Private method with the logic
     */
    _addNote(valueFields, modelFields) {
        return __awaiter(this, void 0, void 0, function* () {
            if (!AnkiDroid.androidCheck())
                return [new Error(Errors.OS_ERROR)];
            const permissionStatus = yield AnkiDroid.checkPermission();
            if (!permissionStatus)
                return [new Error(Errors.PERMISSION_ERROR)];
            const noteErrorCheckResults = this.checkForAddNoteErrors({
                valueFields,
                modelFields,
            });
            if (noteErrorCheckResults)
                return [new Error(noteErrorCheckResults)];
            let deckPropertiesToUse = {};
            let modelPropertiesToUse = {};
            let { deckId, modelId } = this.settings;
            const { deckProperties, modelProperties } = this.settings;
            const deckIdAndProperties = this.getValidIdAndProperties(deckId, deckProperties);
            if (deckIdAndProperties instanceof Error) {
                return [deckIdAndProperties];
            }
            else {
                deckId = deckIdAndProperties[0];
                deckPropertiesToUse = deckIdAndProperties[1];
            }
            const modelIdAndProperties = this.getValidIdAndProperties(modelId, modelProperties);
            if (modelIdAndProperties instanceof Error) {
                return [modelIdAndProperties];
            }
            else {
                modelId = modelIdAndProperties[0];
                modelPropertiesToUse = modelIdAndProperties[1];
            }
            // destructure with default values
            const { fields, cardNames, questionFormat, answerFormat, tags = null, css = null, } = modelPropertiesToUse;
            const modelName = modelPropertiesToUse.name;
            const dbModelReference = modelPropertiesToUse.dbReference;
            const deckName = deckPropertiesToUse.name;
            const dbDeckReference = deckPropertiesToUse.dbReference;
            if (!modelId) {
                const errorCheckModelFields = this.checkIfModelFieldsAreTheSame(fields, modelFields);
                if (errorCheckModelFields)
                    return [new Error(errorCheckModelFields)];
            }
            let addedNoteId;
            try {
                addedNoteId = yield AnkiDroidModule.addNote(deckName, deckId, modelName, modelId, dbDeckReference, dbModelReference, modelFields, valueFields, tags, cardNames, questionFormat, answerFormat, css);
            }
            catch (error) {
                console.warn(MODULE_NAME, error.toString());
                return [new Error(Errors.UNKNOWN_ERROR)];
            }
            try {
                const addedNoteIdInt = Number(addedNoteId);
                // check if we received
                if (!addedNoteId || isNaN(addedNoteIdInt)) {
                    console.warn(MODULE_NAME, addedNoteId);
                    // return the appropriate error
                    return [new Error(Errors[addedNoteId])];
                }
                return [null, addedNoteId];
            }
            catch (error) {
                console.warn(MODULE_NAME, error.toString());
                return [new Error(Errors.UNKNOWN_ERROR)];
            }
        });
    }
    ////////////
    // PUBLIC //
    ////////////
    /**
     * Create notes using the created deck model
     * @param valueFields length must match `modelFields`
     * @param modelFields length must match the settings used when creating the deck
     * @return a tuple of any errors and the result `[error, result]`
     * @return the added note ID
     */
    addNote(valueFields, modelFields) {
        return __awaiter(this, void 0, void 0, function* () {
            const [error, response] = yield this._addNote(valueFields, modelFields);
            if (error) {
                return yield this._addNote(valueFields, modelFields);
            }
            return [error, response];
        });
    }
}
//# sourceMappingURL=ankidroid.js.map