var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : new P(function (resolve) { resolve(result.value); }).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
import { NativeModules, PermissionsAndroid, Platform, } from "react-native";
import { Errors, ErrorText, MODULE_NAME, PermissionResults } from "./types";
const { AnkiDroidModule } = NativeModules;
/**
 * Check if android
 * - display error message if not android
 * @returns `true` if android
 */
export function androidCheck() {
    if (Platform.OS === "android") {
        return true;
    }
    console.warn(MODULE_NAME, Errors.OS_ERROR);
    return false;
}
/**
 * Get the AnkiDroid API permission name
 */
export function getPermissionName() {
    return __awaiter(this, void 0, void 0, function* () {
        let permissionName;
        try {
            permissionName = yield AnkiDroidModule.getPermissionName();
        }
        catch (error) {
            permissionName = null;
            console.warn(MODULE_NAME, ErrorText.PERMISSION_NAME, error);
        }
        return permissionName;
    });
}
/**
 * Check if the AnkiDroid API is available on the phone
 * @return `true` if the API is available to use
 */
export function isApiAvailable() {
    return __awaiter(this, void 0, void 0, function* () {
        if (!androidCheck())
            return false;
        let apiAvailable;
        try {
            apiAvailable = yield AnkiDroidModule.isApiAvailable();
        }
        catch (error) {
            apiAvailable = false;
            console.warn(MODULE_NAME, ErrorText.API, error);
        }
        return apiAvailable;
    });
}
/**
 * Check the permission status
 * @return `true` if permission have been granted
 */
export function checkPermission() {
    return __awaiter(this, void 0, void 0, function* () {
        if (!androidCheck())
            return false;
        let permissionName;
        try {
            permissionName = yield getPermissionName();
        }
        catch (error) {
            console.warn(MODULE_NAME, error.toString());
            return null;
        }
        if (!permissionName)
            return false;
        let permission;
        try {
            permission = yield PermissionsAndroid.check(permissionName);
        }
        catch (error) {
            permission = error;
            console.warn(MODULE_NAME, ErrorText.PERMISSIONS_CHECK, error);
        }
        return permission;
    });
}
/**
 * Request AnkiDroid API permissions
 * @param rationale optional `PermissionsAndroid` message to show when requesting permissions
 * @param returns optional `PermissionsAndroid` message to show when requesting permissions
 */
export function requestPermission(rationale = null) {
    return __awaiter(this, void 0, void 0, function* () {
        if (!androidCheck())
            return PermissionResults.DENIED;
        let permissionName;
        try {
            permissionName = yield getPermissionName();
        }
        catch (error) {
            console.warn(MODULE_NAME, error.toString());
            return null;
        }
        if (!permissionName)
            return PermissionResults.DENIED;
        let permissionRequest;
        try {
            permissionRequest = yield PermissionsAndroid.request(permissionName, rationale);
        }
        catch (error) {
            permissionRequest = error.toString();
            console.warn(MODULE_NAME, ErrorText.PERMISSIONS_REQUEST, error);
        }
        return permissionRequest;
    });
}
//# sourceMappingURL=utilities.js.map